import numpy as np
import numpy.linalg as la
import scipy.special as sps
from scipy.optimize import fsolve

# index convention
E = 0
I = 1
X = 2

EE = 0
EI = 1
II = 2
EX = 3
IX = 4


class ginzburg:


    def __init__(self, N, J, K, m_x, theta, sigma_noise, omega , h0):
        self.N = N              #Probably N[E] is the number of excitatory, N[I] the number of inhibitory and N[X] the number of other neurons. Or something...
        self.J = J
        self.K = K
        self.m_x = m_x
        self.theta = theta
        self.omega = omega
        self.h0    = h0
        self.omega_kHz = omega/1000.    #nest measures the frequency in Hz, not in kHz!
        self.sigma_noise = sigma_noise

    def mu_sigma_from_m_corr(self, m, c):
        """calculate the mean and variance of the input to a neuron
        given the mean activity and covariances
        mu_x: mean of noise input
        sigma_x: std of noise input"""
        m_all = np.hstack((m, self.m_x))
        C = np.array([ [ c[EE], c[EI], c[EX] ],
                       [ c[EI], c[II], c[IX] ],
                       [ c[EX], c[IX], 0.    ] ])

        # vector of variances
        a = m_all*(1. - m_all)
        KJ = self.K*self.J          
        KJ2 = self.K*self.J*self.J
        
        mu = np.dot(KJ,m_all)           #Matrix product (formula (17))
        # take into account correlations in the input to determine variance
        sigma2_m = np.dot(KJ2, a)                               # Formula (18)
        sigma2_c = np.diag( np.dot( np.dot(KJ, C), KJ.T ))      # don't neglect correlations (different to the script)
        
        for ii in range(0,len(sigma2_m)):
            if(sigma2_m[ii]+ sigma2_c[ii] <0):
                print 'sigma2_m[', ii, ']=', sigma2_m[ii]
                print 'sigma2_c[', ii, ']=', sigma2_c[ii]
        
        sigma2 = np.sqrt(sigma2_m + sigma2_c + self.sigma_noise**2)
        return mu, sigma2

    def susceptibility_corr(self, m, c, h = 0.):

        mu, sigma = self.mu_sigma_from_m_corr(m, c)
        mu = mu + h
        
        S = 1./(np.sqrt(2.*np.pi)*sigma) * np.exp( -(mu-self.theta)**2 / (2.*sigma**2) ) #Formula (19)
	# assumes a (near-)Heaviside gain function
        return S


    def correlations(self, m, S):
        """solve the linear system of equations determining the correlations
        given we know the susceptibility"""
        W = ((self.J*self.K).T * S).T          

        N = self.N

        a = m*(1.-m)
        c = np.zeros(5, dtype=float)
        
        pref = a[X]/self.N[X] / ( (2.-W[E,E])*(2.-W[I,I]) - W[E,I]*W[I,E] )   
        c[EX] = pref * ( (2. - W[I,I])*W[E,X] + W[E,I]*W[I,X] )
        c[IX] = pref * ( (2. - W[E,E])*W[I,X] + W[I,E]*W[E,X] )

        M = np.matrix([[1. - W[E,E], - W[E,I], 0.],
                       [-0.5*W[I,E], 1. - 0.5*(W[E,E]+W[I,I]), -0.5*W[E,I]],
                       [0., -W[I,E], 1. - W[I,I]]])

        b = np.array([W[E,E]*a[E]/N[E], 0.5*(W[I,E]*a[E]/N[E] + W[E,I]*a[I]/N[I]), W[I,I]*a[I]/N[I]])
        b1 = np.array( [W[E,X]*c[EX], 0.5*(W[E,X]*c[IX] + W[I,X]*c[EX]), W[I,X]*c[IX]] )
        c[:EX] = np.linalg.solve(M, b + b1)    
        return c


    def solve_self_consistently_corr(self, m0, h=0.):
        """determine self-consistent rates and correlations
        taking effect of correlations on input statistics into account"""

        c = np.zeros(5)
        
        def m_zero_corr(m):
            
            mu, sigma = self.mu_sigma_from_m_corr(m, c)
            mu = mu + h
            new_m = 0.5*(1-sps.erf(-(mu-self.theta)/(np.sqrt(2.)*sigma)))

            return new_m - m
            
        err = 1.
        while err > 1e-15:          #Solves (7,8) self-consistently        
            m0 = fsolve(m_zero_corr, m0)
            
            S = self.susceptibility_corr(m0, c, h)
            c_new = self.correlations(np.hstack((m0, self.m_x)), S)
            err = la.norm(c_new - c)
            c = c_new
        return m0, c_new 
        
    def effective_connectivity(self, m0):
        """" determine effective connectivity and its eigenvalues and eigen-
        vectors considering the susceptibility depending on mean activities and
        pairwise correlations"""
        
        m, c = self.solve_self_consistently_corr(m0)
        S = self.susceptibility_corr(m, c)
        W = np.zeros((3,3), dtype=float)        
        W[:2,:] = np.dot(np.diag(S),self.K*self.J)
        
        lambdas, U = np.linalg.eig(W)
        
        return W, lambdas, U
        
    def symmetrize_transform_eigen(self, A, U_inv):
        B = A + A.T                      # Make the right side symmetrical
        B = np.dot(U_inv,np.dot(B,U_inv.T))
        return B            
     
    def M1_linear_perturb_theory(self, m0, tau):
        """Calculate the deviation of the mean activity from the stationary state in dependance of the time t"""
        N_m = np.size(m0)        
        W = np.zeros((3,3), dtype=float) # multiply each row in effective linearized connectivity by corresponding susceptibility S
        m0, c = self.solve_self_consistently_corr(m0)
        mu0, sigma0 = self.mu_sigma_from_m_corr(m0, c)

        S = self.susceptibility_corr(m0, c)     #S gives a vector in R^2
               
        W[0:2,:] = ((self.J*self.K).T * S).T  
        W[2, :] = np.zeros(3)       #W_XE = W_XI = W_XX = 0        
        
        lambdas, U = np.linalg.eig(W) # U=(u_1,u_2,...), u_i are the eigen vectors of W
        U_inv = np.linalg.inv(U)
        S_vec = np.array([S[0],S[1],0.])
        UinvS_vec = np.dot(U_inv,S_vec) #Direction of the perturbation in the eigen system of the ODE

        M1_eigen = np.zeros(3, dtype=complex)        
        for k in range(0,N_m+1):
            M1_eigen[k] = self.h0 * UinvS_vec[k] * (-1j * tau * self.omega_kHz +1 - lambdas[k] ) /((tau*self.omega_kHz)**2+(lambdas[k]-1)**2)
            
        M1 = np.dot(U,M1_eigen)       
        return M1, M1_eigen, lambdas    
    
    def C1_linear_perturb_theory(self, m0_no_x, tau):
        """Calculates the solution of the linearized ODE for the deviation of the correlations from the stationary solution.
        t is actually assumed to be a list of time values."""
        N_m = np.size(m0_no_x)+1
        
        m0_no_x, c0_vec = self.solve_self_consistently_corr(m0_no_x)        
        S = self.susceptibility_corr(m0_no_x, c0_vec)
        
        m0 = np.hstack((m0_no_x, self.m_x))
        a0 = m0*(1.-m0)
        a0_diag = np.diag(a0/self.N)
        c0 = np.array([ [ c0_vec[EE], c0_vec[EI], c0_vec[EX] ],
                        [ c0_vec[EI], c0_vec[II], c0_vec[IX] ],
                        [ c0_vec[EX], c0_vec[IX], 0.    ] ])        
        
        mu, sigma = self.mu_sigma_from_m_corr(m0_no_x, c0_vec)

        
        M1, M1_eigen, lambdas_from_calc_M1 = self.M1_linear_perturb_theory(m0_no_x, tau)        
        
        Z = np.zeros((3,3), dtype=float)        
        V = np.zeros((3,3), dtype=float)        
        W = np.zeros((3,3), dtype=float)        
        
        Z[0:2,:] = self.J*self.K #Z_XE = Z_XI = Z_XX = 0
        V[0:2,:] = ((self.J*self.K).T *(self.theta-mu)/sigma**2 * S).T #V_XE = V_XI = V_XX = 0   
        W[0:2,:] = ((self.J*self.K).T * S).T #W_XE = W_XI = W_XX = 0   
               
        if(np.linalg.matrix_rank(W)!=1):
            lambdas, U = np.linalg.eig(W)            # U=(u_1,u_2,...), u_i are the eigen vectors of W
        else:   # Also always holds W[i,0] = W[i,2] forall i and W[0,j] = W[1,j] forall j
            #print 'rank(W)=1'
            lambdas = np.zeros(3, dtype = complex)            
            lambdas[0] = W[0,0]+W[0,1]
            U = np.zeros((3,3), dtype = complex)
            U[:,0] = 1./np.sqrt(2.) * np.array([1.,1.,0.])
            U[:,1] = 1./np.sqrt(W[0,0]**2 + W[0,1]**2) * np.array([-W[0,1], W[0,0], 0. ])
            U[:,2] = 1./np.sqrt(2.) * np.array([1.,0.,-1.])    
        U_inv = np.linalg.inv(U)
        
        Vc_plus_a = np.dot(V,a0_diag + c0)
        prefactor_delta_c_eigen = np.zeros((N_m,N_m), dtype =complex)        
        for ii in range(0,N_m):
            for jj in range(0,N_m):
                prefactor_delta_c_eigen[ii,jj]  = (1j * self.omega_kHz * tau + 2. - lambdas[ii]-lambdas[jj])

        right_side_natural_basis = (np.dot(np.diag(np.dot(Z,M1)), Vc_plus_a) 
                                    + np.dot(W,np.dot(np.diag((1.-2.*m0)/self.N), np.diag(M1))) 
                                    + self.h0 * Vc_plus_a)
        right_side_eigen = self.symmetrize_transform_eigen(right_side_natural_basis, U_inv) # Make the right side symmetrical and transform into eigensystem

        C1_eigen = np.zeros((N_m,N_m), dtype=complex)
        for ii in range(0,N_m):
            for jj in range(0,N_m):        
                C1_eigen[ii,jj] = right_side_eigen[ii,jj]/prefactor_delta_c_eigen[ii,jj]
        
        C1 = np.dot(np.dot(U,C1_eigen),np.transpose(U))
        
        ################################################
        ### Calculate every contribution seperately ####
        ################################################        
        
        right_side_natural_basis_S_m = np.dot(np.diag(np.dot(Z,M1)), Vc_plus_a)
        right_side_eigen_S_m = self.symmetrize_transform_eigen(right_side_natural_basis_S_m, U_inv) # Make the right side symmetrical and transform into eigensystem
        
        right_side_natural_basis_S_h = self.h0 * Vc_plus_a
        right_side_eigen_S_h = self.symmetrize_transform_eigen(right_side_natural_basis_S_h, U_inv) # Make the right side symmetrical and transform into eigensystem
        
        right_side_natural_basis_a_m = np.dot(W,np.dot(np.diag((1.-2.*m0)/self.N), np.diag(M1)))
        right_side_eigen_a_m = self.symmetrize_transform_eigen(right_side_natural_basis_a_m, U_inv) # Make the right side symmetrical and transform into eigensystem
        
        
        Proj_first = np.zeros((3,3), dtype=float)
        Proj_first[0,0] = 1.
        Proj_second = np.zeros((3,3), dtype=float)
        Proj_second[1,1] = 1.        
        
        M1_first = np.diag(np.dot(U,np.dot(Proj_first,np.dot(U_inv,M1)))) #Project M1 on the first eigen direction
        M1_second = np.diag(np.dot(U,np.dot(Proj_second,np.dot(U_inv,M1)))) #Project M1 on the second eigen direction        
        
        right_side_natural_basis_a_m_first = np.dot(W,np.dot(np.diag((1.-2.*m0)/self.N), M1_first))
        right_side_eigen_a_m_first = self.symmetrize_transform_eigen(right_side_natural_basis_a_m_first, U_inv) # Make the right side symmetrical and transform into eigensystem
        
        right_side_natural_basis_a_m_second = np.dot(W,np.dot(np.diag((1.-2.*m0)/self.N), M1_second))
        right_side_eigen_a_m_second = self.symmetrize_transform_eigen(right_side_natural_basis_a_m_second, U_inv) # Make the right side symmetrical and transform into eigensystem
        
        C1_eigen_S_m = np.zeros((N_m,N_m), dtype=complex)
        C1_eigen_S_h = np.zeros((N_m,N_m), dtype=complex)      
        C1_eigen_a_m = np.zeros((N_m,N_m), dtype=complex)
        C1_eigen_a_m_first = np.zeros((N_m,N_m), dtype=complex)
        C1_eigen_a_m_second = np.zeros((N_m,N_m), dtype=complex)
        for ii in range(0,N_m):
            for jj in range(0,N_m):        
                C1_eigen_S_m[ii,jj] = right_side_eigen_S_m[ii,jj]/prefactor_delta_c_eigen[ii,jj]
                C1_eigen_S_h[ii,jj] = right_side_eigen_S_h[ii,jj]/prefactor_delta_c_eigen[ii,jj]
                C1_eigen_a_m[ii,jj] = right_side_eigen_a_m[ii,jj]/prefactor_delta_c_eigen[ii,jj]
                
                C1_eigen_a_m_first[ii,jj] = right_side_eigen_a_m_first[ii,jj]/prefactor_delta_c_eigen[ii,jj]
                C1_eigen_a_m_second[ii,jj] = right_side_eigen_a_m_second[ii,jj]/prefactor_delta_c_eigen[ii,jj]

        
        C1_S_m = np.dot(np.dot(U,C1_eigen_S_m),np.transpose(U))
        C1_S_h = np.dot(np.dot(U,C1_eigen_S_h),np.transpose(U))
        C1_a_m = np.dot(np.dot(U,C1_eigen_a_m),np.transpose(U))
        
        return C1, C1_eigen, C1_eigen_S_m, C1_eigen_S_h, C1_eigen_a_m, C1_eigen_a_m_first, C1_eigen_a_m_second, C1_S_m, C1_S_h, C1_a_m
        
        
    def mean_response_step_current(self, m_exp, tau, t, turn_on):
        """Calculate the deviation of the mean activity from the stationary state in dependance of the time t as a response to a step-shaped perturbation"""
        m0, c0 = self.solve_self_consistently_corr(m_exp)
        S0     = self.susceptibility_corr(m0, c0)     #S0 is a vector in R^2        
        
        W        = np.zeros((3,3), dtype=float)        
        W[0:2,:] = ((self.J*self.K).T * S0).T #W_XE = W_XI = W_XX = 0    
        
        lambdas, U = np.linalg.eig(W)
        U_inv = np.linalg.inv(U)
        
        kernel = np.linalg.inv(np.identity(3) - W)        
        exp_kernel = np.diag(np.exp((lambdas - 1.)*t/tau))
        if(turn_on == True):        
            E = np.identity(3) - np.dot(np.dot(U, exp_kernel), U_inv)
            coeff_del_m_hom   = -self.h0 * np.dot(np.dot(U_inv,kernel),np.array([S0[0],S0[1],0.]))
            coeff_del_m_const = - coeff_del_m_hom
        else:
            E = np.dot(np.dot(U,exp_kernel), U_inv)
            coeff_del_m_hom   = self.h0 * np.dot(np.dot(U_inv,kernel),np.array([S0[0],S0[1],0.]))
            coeff_del_m_const = np.zeros_like(coeff_del_m_hom) 
        
        del_m_total = self.h0 * np.dot(np.dot(E, kernel),np.array([S0[0],S0[1],0.]))
        
        return del_m_total, coeff_del_m_hom, coeff_del_m_const
    
    def mean_response_step_current_simple(self, m_exp, tau, t, turn_on):
        """Calculate the deviation of the mean activity from the stationary state in dependance of the time t as a response to a step-shaped perturbation"""
        m0, c0 = self.solve_self_consistently_corr(m_exp)
        S0     = self.susceptibility_corr(m0, c0)     #S0 is a vector in R^2        
        
        W        = np.zeros((3,3), dtype=float)        
        W[0:2,:] = ((self.J*self.K).T * S0).T #W_XE = W_XI = W_XX = 0    
        
        lambdas, U = np.linalg.eig(W)
        U_inv = np.linalg.inv(U)
        
        kernel = np.linalg.inv(np.identity(3) - W)        
        exp_kernel = np.diag(np.exp((lambdas - 1.)*t/tau))
        if(turn_on == True):        
            E = np.identity(3) - np.dot(np.dot(U, exp_kernel), U_inv)
        else:
            E = np.dot(np.dot(U,exp_kernel), U_inv)
            
        coeff_del_m_hom   = self.h0 * np.dot(np.dot(U_inv,kernel),np.array([S0[0],S0[1],0.]))
        
        del_m_total = self.h0 * np.dot(np.dot(E, kernel),np.array([S0[0],S0[1],0.]))
        
        return del_m_total, coeff_del_m_hom    
    
    def corr_response_step_current(self, m_exp, tau, t, turn_on):
        """Calculate the deviation of the pairwise correlations from the stationary state in dependance of the time t as a response to a step-shaped perturbation"""
        m0_loc, c0_vec = self.solve_self_consistently_corr(m_exp)
        m0             = np.hstack((m0_loc, self.m_x))  
        S0             = self.susceptibility_corr(m0_loc, c0_vec)     #S0 is a vector in R^2        
        mu0, sigma0    = self.mu_sigma_from_m_corr(m0_loc, c0_vec)
                
        a0      = m0*(1.-m0)
        a0_diag = np.diag(a0/self.N)
        c0      = np.array([ [ c0_vec[EE], c0_vec[EI], c0_vec[EX] ],
                        [ c0_vec[EI], c0_vec[II], c0_vec[IX] ],
                        [ c0_vec[EX], c0_vec[IX], 0.    ] ])        
                
        Z = np.zeros((3,3), dtype=float)        
        V = np.zeros((3,3), dtype=float)        
        W = np.zeros((3,3), dtype=float)        
        
        Z[0:2,:] = self.J*self.K 
        V[0:2,:] = ((self.J*self.K).T *(self.theta-mu0)/sigma0**2 * S0).T                    
        W[0:2,:] = ((self.J*self.K).T * S0).T #W_XE = W_XI = W_XX = 0    
        
        if(np.linalg.matrix_rank(W)!=1):
            lambdas, U = np.linalg.eig(W)            # U=(u_1,u_2,...), u_i are the eigen vectors of W
        else:   # Is also always holds W[i,0] = W[i,2] forall i and W[0,j] = W[1,j] forall j
            #print 'rank(W)=1'
            lambdas    = np.zeros(3, dtype = complex)            
            lambdas[0] = W[0,0]+W[0,1]
            U          = np.zeros((3,3), dtype = complex)
            U[:,0]     = 1./np.sqrt(2.) * np.array([1.,1.,0.])
            U[:,1]     = 1./np.sqrt(W[0,0]**2 + W[0,1]**2) * np.array([-W[0,1], W[0,0], 0. ])
            U[:,2]     = 1./np.sqrt(2.) * np.array([1.,0.,-1.])
     
        U_inv = np.linalg.inv(U)
        v1      = U[:,0]
        v2      = U[:,1]
        lambda1 = lambdas[0]
        lambda2 = lambdas[1]
        Vc_plus_a = np.dot(V,a0_diag + c0)
        del_m_total, coeff_del_m_hom, coeff_del_m_const = self.mean_response_step_current(m_exp, tau, t, turn_on)
        
        del_m_0 = coeff_del_m_const[0] * v1 + coeff_del_m_const[1] * v2 #constant part
        del_m_1 = coeff_del_m_hom[0] * v1
        del_m_2 = coeff_del_m_hom[1] * v2
        
        # Inhomogeneities for delta_c coming from the inhomogeneous (constant) solution for delta_m
        inhom_0_natural_basis_S_m = np.dot(np.diag(np.dot(Z,del_m_0)), Vc_plus_a)
        inhom_0_natural_basis_S_m = inhom_0_natural_basis_S_m + np.transpose(inhom_0_natural_basis_S_m)                      # Make the right side symmetrical
        inhom_0_eigen_S_m         = np.dot(U_inv,np.dot(inhom_0_natural_basis_S_m,np.transpose(U_inv)))
        
        if(turn_on == True):
            inhom_0_natural_basis_S_h = self.h0 * Vc_plus_a
            inhom_0_natural_basis_S_h = inhom_0_natural_basis_S_h + np.transpose(inhom_0_natural_basis_S_h)                      # Make the right side symmetrical
            inhom_0_eigen_S_h         = np.dot(U_inv,np.dot(inhom_0_natural_basis_S_h,np.transpose(U_inv)))
        else:
            inhom_0_eigen_S_h = np.zeros((3,3), dtype=float)
            
        inhom_0_natural_basis_a_m = np.dot(W,np.dot(np.diag((1.-2.*m0)/self.N), np.diag(del_m_0)))
        inhom_0_natural_basis_a_m = inhom_0_natural_basis_a_m + np.transpose(inhom_0_natural_basis_a_m)                      # Make the right side symmetrical
        inhom_0_eigen_a_m         = np.dot(U_inv,np.dot(inhom_0_natural_basis_a_m,np.transpose(U_inv)))
        
        # Inhomogeneities for delta_c coming from the homogeneous solution for delta_m
        inhom_1_natural_basis_S_m = np.dot(np.diag(np.dot(Z,del_m_1)), Vc_plus_a)
        inhom_1_natural_basis_S_m = inhom_1_natural_basis_S_m + np.transpose(inhom_1_natural_basis_S_m)                      # Make the right side symmetrical
        inhom_1_eigen_S_m         = np.dot(U_inv,np.dot(inhom_1_natural_basis_S_m,np.transpose(U_inv)))
        
        inhom_1_natural_basis_a_m = np.dot(W,np.dot(np.diag((1.-2.*m0)/self.N), np.diag(del_m_1)))
        inhom_1_natural_basis_a_m = inhom_1_natural_basis_a_m + np.transpose(inhom_1_natural_basis_a_m)                      # Make the right side symmetrical
        inhom_1_eigen_a_m         = np.dot(U_inv,np.dot(inhom_1_natural_basis_a_m,np.transpose(U_inv)))
        
        inhom_2_natural_basis_S_m = np.dot(np.diag(np.dot(Z,del_m_2)), Vc_plus_a)
        inhom_2_natural_basis_S_m = inhom_2_natural_basis_S_m + np.transpose(inhom_2_natural_basis_S_m)                      # Make the right side symmetrical
        inhom_2_eigen_S_m         = np.dot(U_inv,np.dot(inhom_2_natural_basis_S_m,np.transpose(U_inv)))
        
        inhom_2_natural_basis_a_m = np.dot(W,np.dot(np.diag((1.-2.*m0)/self.N), np.diag(del_m_2)))
        inhom_2_natural_basis_a_m = inhom_2_natural_basis_a_m + np.transpose(inhom_2_natural_basis_a_m)                      # Make the right side symmetrical
        inhom_2_eigen_a_m         = np.dot(U_inv,np.dot(inhom_2_natural_basis_a_m,np.transpose(U_inv)))
        
        #Now, stick everything together:
        del_c_eigen_S_m = inhom_0_eigen_S_m + inhom_1_eigen_S_m * np.exp((-1. + lambda1) * t/tau) + inhom_2_eigen_S_m * np.exp((-1. + lambda2) * t/tau) 
        del_c_eigen_S_h = inhom_0_eigen_S_h
        del_c_eigen_a_m = inhom_0_eigen_a_m + inhom_1_eigen_a_m * np.exp((-1. + lambda2) * t/tau) + inhom_2_eigen_a_m * np.exp((-1. + lambda2) * t/tau)
        #Choose prefactor of the homogeneous part such that del_c(t=0) = 0
        if(turn_on == True):        
            prefactor_hom_del_c_eigen = - (inhom_0_eigen_S_m + inhom_0_eigen_S_h + inhom_0_eigen_a_m + inhom_1_eigen_S_m + inhom_1_eigen_a_m + inhom_2_eigen_S_m + inhom_2_eigen_a_m)
        else:
            res1, res2, coeff_del_m_const_turn_on = self.mean_response_step_current(m_exp, tau, 0., True)
            del_m_0_turn_on = coeff_del_m_const_turn_on[0] * v1 + coeff_del_m_const_turn_on[1] * v2 #constant part
 
            inhom_0_natural_basis_S_m_turn_on = np.dot(np.diag(np.dot(Z,del_m_0_turn_on)), Vc_plus_a)
            inhom_0_natural_basis_S_m_turn_on = inhom_0_natural_basis_S_m_turn_on + np.transpose(inhom_0_natural_basis_S_m_turn_on)                      # Make the right side symmetrical
            inhom_0_eigen_S_m_turn_on         = np.dot(U_inv,np.dot(inhom_0_natural_basis_S_m_turn_on,np.transpose(U_inv)))
            
            inhom_0_natural_basis_S_h_turn_on = self.h0 * Vc_plus_a
            inhom_0_natural_basis_S_h_turn_on = inhom_0_natural_basis_S_h_turn_on + np.transpose(inhom_0_natural_basis_S_h_turn_on)                      # Make the right side symmetrical
            inhom_0_eigen_S_h_turn_on         = np.dot(U_inv,np.dot(inhom_0_natural_basis_S_h_turn_on,np.transpose(U_inv)))
                
            inhom_0_natural_basis_a_m_turn_on = np.dot(W,np.dot(np.diag((1.-2.*m0)/self.N), np.diag(del_m_0_turn_on)))
            inhom_0_natural_basis_a_m_turn_on = inhom_0_natural_basis_a_m_turn_on + np.transpose(inhom_0_natural_basis_a_m_turn_on)                      # Make the right side symmetrical
            inhom_0_eigen_a_m_turn_on         = np.dot(U_inv,np.dot(inhom_0_natural_basis_a_m_turn_on,np.transpose(U_inv)))            
            
            prefactor_hom_del_c_eigen = -(inhom_1_eigen_S_m + inhom_1_eigen_a_m + inhom_2_eigen_S_m + inhom_2_eigen_a_m) + inhom_0_eigen_S_m_turn_on + inhom_0_eigen_S_h_turn_on + inhom_0_eigen_a_m_turn_on 
            
            if(t==0):
                print 'minus part prefactor', inhom_1_eigen_S_m + inhom_1_eigen_a_m + inhom_2_eigen_S_m + inhom_2_eigen_a_m
                print 'inhomogeneous part', del_c_eigen_S_m + del_c_eigen_S_h + del_c_eigen_a_m
                
        del_c_eigen = np.zeros((len(m0),len(m0)), dtype=float)  
        for ii in range(0,len(m0)):
            for jj in range(0,len(m0)):
                del_c_eigen[ii,jj] = prefactor_hom_del_c_eigen[ii,jj] * np.exp((-2.+lambdas[ii] + lambdas[jj]) * t/tau) 
        
        del_c_eigen = del_c_eigen + del_c_eigen_S_m + del_c_eigen_S_h + del_c_eigen_a_m
        
        del_c_S_m = np.dot(np.dot(U,del_c_eigen_S_m),np.transpose(U))
        del_c_S_h = np.dot(np.dot(U,del_c_eigen_S_h),np.transpose(U))
        del_c_a_m = np.dot(np.dot(U,del_c_eigen_a_m),np.transpose(U))        
        del_c     = np.dot(np.dot(U,del_c_eigen),np.transpose(U))
        
        if(turn_on == True):
            return del_c, del_c_S_m, del_c_S_h, del_c_a_m, np.dot(np.dot(U,inhom_0_eigen_S_m + inhom_0_eigen_S_h + inhom_0_eigen_a_m),np.transpose(U))
        else:
            return del_c, del_c_S_m, del_c_S_h, del_c_a_m, np.dot(np.dot(U,inhom_0_eigen_S_m_turn_on + inhom_0_eigen_S_h_turn_on + inhom_0_eigen_a_m_turn_on),np.transpose(U)) 

    
    def corr_response_step_current_simple(self, m_exp, tau, t, turn_on):
        """Calculate the deviation of the pairwise correlations from the stationary state in dependance of the time t as a response to a step-shaped perturbation"""
        m0_loc, c0_vec = self.solve_self_consistently_corr(m_exp)
        m0             = np.hstack((m0_loc, self.m_x))  
        S0             = self.susceptibility_corr(m0_loc, c0_vec)     #S0 is a vector in R^2        
        mu0, sigma0    = self.mu_sigma_from_m_corr(m0_loc, c0_vec)
                
        a0      = m0*(1.-m0)
        a0_diag = np.diag(a0/self.N)
        c0      = np.array([ [ c0_vec[EE], c0_vec[EI], c0_vec[EX] ],
                        [ c0_vec[EI], c0_vec[II], c0_vec[IX] ],
                        [ c0_vec[EX], c0_vec[IX], 0.    ] ])        
                
        Z = np.zeros((3,3), dtype=float)        
        V = np.zeros((3,3), dtype=float)        
        W = np.zeros((3,3), dtype=float)        
        
        Z[0:2,:] = self.J*self.K 
        V[0:2,:] = ((self.J*self.K).T *(self.theta-mu0)/sigma0**2 * S0).T                    
        W[0:2,:] = ((self.J*self.K).T * S0).T #W_XE = W_XI = W_XX = 0    
        
        if(np.linalg.matrix_rank(W)!=1):
            lambdas, U = np.linalg.eig(W)            # U=(u_1,u_2,...), u_i are the eigen vectors of W
        else:   # Is also always holds W[i,0] = W[i,2] forall i and W[0,j] = W[1,j] forall j
            lambdas    = np.zeros(3, dtype = complex)            
            lambdas[0] = W[0,0]+W[0,1]
            U          = np.zeros((3,3), dtype = complex)
            U[:,0]     = 1./np.sqrt(2.) * np.array([1.,1.,0.])
            U[:,1]     = 1./np.sqrt(W[0,0]**2 + W[0,1]**2) * np.array([-W[0,1], W[0,0], 0. ])
            U[:,2]     = 1./np.sqrt(2.) * np.array([1.,0.,-1.])
     
        U_inv   = np.linalg.inv(U)
        v1      = U[:,0]
        v2      = U[:,1]
        lambda1 = lambdas[0]
        lambda2 = lambdas[1]
        Vc_plus_a = np.dot(V,a0_diag + c0)
        del_m_total, coeff_del_m_hom = self.mean_response_step_current_simple(m_exp, tau, t, turn_on)
        
        if(turn_on == True):
            del_m_0 = coeff_del_m_hom[0] * v1 + coeff_del_m_hom[1] * v2 #constant part
            del_m_1 = -coeff_del_m_hom[0] * v1
            del_m_2 = -coeff_del_m_hom[1] * v2
        else:
            del_m_0 = np.zeros_like(v1)
            del_m_1 = coeff_del_m_hom[0] * v1
            del_m_2 = coeff_del_m_hom[1] * v2
        
        # Inhomogeneities for delta_c coming from the inhomogeneous (constant) solution for delta_m
        if(turn_on == True):
            inhom_0_eigen_S_m = self.symmetrize_transform_eigen(np.dot(np.diag(np.dot(Z,del_m_0)), Vc_plus_a), U_inv)
            inhom_0_eigen_S_h = self.symmetrize_transform_eigen(self.h0 * Vc_plus_a, U_inv)
            inhom_0_eigen_a_m = self.symmetrize_transform_eigen(np.dot(W,np.dot(np.diag((1.-2.*m0)/self.N), np.diag(del_m_0))), U_inv)
        else:
            inhom_0_eigen_S_m = np.zeros((3,3), dtype=float)
            inhom_0_eigen_S_h = np.zeros((3,3), dtype=float)
            inhom_0_eigen_a_m = np.zeros((3,3), dtype=float)
            
        # Inhomogeneities for delta_c coming from the homogeneous solution for delta_m
        inhom_1_eigen_S_m = self.symmetrize_transform_eigen(np.dot(np.diag(np.dot(Z,del_m_1)), Vc_plus_a), U_inv)
        inhom_1_eigen_a_m = self.symmetrize_transform_eigen(np.dot(W,np.dot(np.diag((1.-2.*m0)/self.N), np.diag(del_m_1))), U_inv)
        
        inhom_2_eigen_S_m = self.symmetrize_transform_eigen(np.dot(np.diag(np.dot(Z,del_m_2)), Vc_plus_a), U_inv)
        inhom_2_eigen_a_m = self.symmetrize_transform_eigen(np.dot(W,np.dot(np.diag((1.-2.*m0)/self.N), np.diag(del_m_2))), U_inv)
        
        prefactor_hom_del_c_eigen = np.zeros((len(m0),len(m0)), dtype=float)        
        del_c_eigen               = np.zeros((len(m0),len(m0)), dtype=float)
        del_c_eigen_S_m           = np.zeros((len(m0),len(m0)), dtype=float)
        del_c_eigen_S_h           = np.zeros((len(m0),len(m0)), dtype=float)
        del_c_eigen_a_m           = np.zeros((len(m0),len(m0)), dtype=float)
        
        if(turn_on == False):
            del_m_0_turn_on = coeff_del_m_hom[0] * v1 + coeff_del_m_hom[1] * v2 #constant part
            inhom_0_eigen_S_m_turn_on = self.symmetrize_transform_eigen(np.dot(np.diag(np.dot(Z,del_m_0_turn_on)), Vc_plus_a), U_inv)
            inhom_0_eigen_S_h_turn_on = self.symmetrize_transform_eigen(self.h0 * Vc_plus_a, U_inv)
            inhom_0_eigen_a_m_turn_on = self.symmetrize_transform_eigen(np.dot(W,np.dot(np.diag((1.-2.*m0)/self.N), np.diag(del_m_0_turn_on))), U_inv) 
        
        for ii in range(0,len(m0)):
            for jj in range(0,len(m0)):

                if(turn_on == True):
                    #Choose prefactor of the homogeneous part such that del_c(t=0) = 0
                    prefactor_hom_del_c_eigen[ii,jj] = - ((inhom_0_eigen_S_m[ii,jj] + inhom_0_eigen_S_h[ii,jj] + inhom_0_eigen_a_m[ii,jj])/(2. - lambdas[ii] - lambdas[jj]) + (inhom_1_eigen_S_m[ii,jj] + inhom_1_eigen_a_m[ii,jj])/(1. + lambda1 - lambdas[ii] - lambdas[jj]) + (inhom_2_eigen_S_m[ii,jj] + inhom_2_eigen_a_m[ii,jj])/(1. + lambda2 - lambdas[ii] - lambdas[jj]))
                else:
                    #Choose prefactor of the homogeneous part such that del_c(t=0) = lim_{t vs infty} c_{turn_on}(t)
                    prefactor_hom_del_c_eigen[ii,jj] = -((inhom_1_eigen_S_m[ii,jj] + inhom_1_eigen_a_m[ii,jj])/(1. + lambda1 - lambdas[ii] - lambdas[jj]) + (inhom_2_eigen_S_m[ii,jj] + inhom_2_eigen_a_m[ii,jj])/(1. + lambda2 - lambdas[ii] - lambdas[jj])) + (inhom_0_eigen_S_m_turn_on[ii,jj] + inhom_0_eigen_S_h_turn_on[ii,jj] + inhom_0_eigen_a_m_turn_on[ii,jj])/(2. - lambdas[ii] - lambdas[jj])                 
                
                del_c_eigen[ii,jj] = prefactor_hom_del_c_eigen[ii,jj] * np.exp((-2.+lambdas[ii] + lambdas[jj]) * t/tau) 
                del_c_eigen_S_m[ii,jj] = inhom_0_eigen_S_m[ii,jj]/(2. - lambdas[ii] - lambdas[jj]) + inhom_1_eigen_S_m[ii,jj] * np.exp((-1. + lambda1) * t/tau)/(1. + lambda1 - lambdas[ii] - lambdas[jj]) + inhom_2_eigen_S_m[ii,jj] * np.exp((-1. + lambda2) * t/tau)/(1. + lambda2 - lambdas[ii] - lambdas[jj]) 
                del_c_eigen_S_h[ii,jj] = inhom_0_eigen_S_h[ii,jj]/(2. - lambdas[ii] - lambdas[jj])
                del_c_eigen_a_m[ii,jj] = inhom_0_eigen_a_m[ii,jj]/(2. - lambdas[ii] - lambdas[jj]) + inhom_1_eigen_a_m[ii,jj] * np.exp((-1. + lambda2) * t/tau)/(1. + lambda1 - lambdas[ii] - lambdas[jj]) + inhom_2_eigen_a_m[ii,jj] * np.exp((-1. + lambda2) * t/tau)/(1. + lambda2 - lambdas[ii] - lambdas[jj])         

        del_c_eigen = del_c_eigen + del_c_eigen_S_m + del_c_eigen_S_h + del_c_eigen_a_m
        
        del_c_S_m = np.dot(np.dot(U,del_c_eigen_S_m),np.transpose(U))
        del_c_S_h = np.dot(np.dot(U,del_c_eigen_S_h),np.transpose(U))
        del_c_a_m = np.dot(np.dot(U,del_c_eigen_a_m),np.transpose(U))        
        del_c     = np.dot(np.dot(U,del_c_eigen),np.transpose(U))
        
        return del_c, del_c_S_m, del_c_S_h, del_c_a_m