
from scipy import *


import matplotlib as mp
import numpy as np
#mp.use('PS')
mp.use('GTKAgg')


from matplotlib.pylab import *


class panel_factory():
  """Class that generates a subpanel in the PloS figure"""
  
  def __init__(self, scale, n_pan_x, n_pan_y, hoffset, voffset, hspace, vspace, panel_width, panel_height):
    self.scale = scale
    self.n_pan_x = n_pan_x
    self.n_pan_y = n_pan_y
    self.voffset = voffset
    self.hoffset = hoffset
    self.voffset = voffset
    self.hspace = hspace
    self.vspace = vspace
    self.panel_width = panel_width
    self.panel_height = panel_height

  def new_empty_panel(self, nx, ny, label):
    """Create new panel with an axes object at position nx, ny"""

    assert(nx >= 0 and nx < self.n_pan_x)
    assert(ny >= 0 and ny < self.n_pan_y)

    pos = [self.hoffset + nx*(self.hspace + self.panel_width),
           self.voffset + (self.n_pan_y-ny-1)*(self.vspace + self.panel_height),
           self.panel_width, self.panel_height]

    ## generate axes object
    #ax = axes(pos)

    ## panel label
    label_pos = [-0.2,0.99] # position of panel label (relative to each subpanel)
    label_fs = self.scale * 12           # fontsize of panel label
    
    text(label_pos[0],label_pos[1], r'\bfseries{}' + label,\
             fontdict={'fontsize' : label_fs,
                       'weight' : 'bold',
                       'horizontalalignment' : 'left',
                       'verticalalignment' : 'bottom'},
         transform = ax.transAxes)

  def new_panel(self, nx, ny, label):
    """Create new panel with an axes object at position nx, ny"""

    assert(nx >= 0 and nx < self.n_pan_x)
    assert(ny >= 0 and ny < self.n_pan_y)

    pos = [self.hoffset + nx*(self.hspace + self.panel_width),
           self.voffset + (self.n_pan_y-ny-1)*(self.vspace + self.panel_height),
           self.panel_width, self.panel_height]

    ## generate axes object
    ax = axes(pos)

    ## panel label
    label_pos = [-0.2,0.99] # position of panel label (relative to each subpanel)
    label_fs = self.scale * 12           # fontsize of panel label
    
    text(label_pos[0],label_pos[1], r'\bfseries{}' + label,\
             fontdict={'fontsize' : label_fs,
                       'weight' : 'bold',
                       'horizontalalignment' : 'left',
                       'verticalalignment' : 'bottom'},
         transform = ax.transAxes)

  def new_empty_panel(self, nx, ny, label, panel_width = False):
    """Create new panel at position nx, ny"""

    assert(nx >= 0 and nx < self.n_pan_x)
    assert(ny >= 0 and ny < self.n_pan_y)
    
    if(panel_width==False):
        panel_width = self.panel_width    

    pos = [self.hoffset + nx*(self.hspace + self.panel_width),
           self.voffset + (self.n_pan_y-ny-1)*(self.vspace + self.panel_height),
           panel_width, self.panel_height]


    ax = axes(pos, frameon = False)
    xticks([])
    yticks([])
    

    ## panel label
    label_pos = [-0.2,0.99] # position of panel label (relative to each subpanel)
    label_fs = self.scale * 12           # fontsize of panel label
    
    text(label_pos[0],label_pos[1], r'\bfseries{}' + label,\
             fontdict={'fontsize' : label_fs,
                       'weight' : 'bold',
                       'horizontalalignment' : 'left',
                       'verticalalignment' : 'bottom'},
              transform = ax.transAxes)

    return pos
 

def create_fig_plos(fig, scale, width, n_horz_panels, n_vert_panels, hoffset=0.1, voffset_total=0.18):
  """Single column PloS figure"""

  panel_wh_ratio = (1. + np.sqrt(5)) / 2. # golden ratio
  
  height = width / panel_wh_ratio * n_vert_panels / n_horz_panels
  
  rcParams['figure.figsize'] = (scale*width, scale*height)
   
  # resolution of figures in dpi
  # does not influence eps output
  rcParams['figure.dpi'] = 300
  
  # font
  rcParams['font.size'] = scale*8
  rcParams['legend.fontsize'] = scale*8
  #rcParams['font.family'] = "helvetica"

  rcParams['lines.linewidth'] = scale*1.0

  # size of markers (points in point plots)
  rcParams['lines.markersize'] = scale * 3.0
  rcParams['patch.linewidth'] = scale * 1.0
  rcParams['axes.linewidth'] = scale * 1.0     # edge linewidth

  # ticks distances
  rcParams['xtick.major.size'] = scale * 4      # major tick size in points
  rcParams['xtick.minor.size'] = scale * 2      # minor tick size in points
  rcParams['lines.markeredgewidth'] = scale * 0.5  # line width of ticks
  rcParams['grid.linewidth'] = scale * 0.5
  rcParams['xtick.major.pad'] = scale * 4      # distance to major tick label in points
  rcParams['xtick.minor.pad'] = scale * 4      # distance to the minor tick label in points
  rcParams['ytick.major.size'] = scale * 4      # major tick size in points
  rcParams['ytick.minor.size'] = scale * 2      # minor tick size in points
  rcParams['ytick.major.pad'] = scale * 4      # distance to major tick label in points
  rcParams['ytick.minor.pad'] = scale * 4      # distance to the minor tick label in points
  
  # ticks textsize
  rcParams['ytick.labelsize'] = scale * 8
  rcParams['xtick.labelsize'] = scale * 8

  # use latex to generate the labels in plots
  # not needed anymore in newer versions
  # using this, font detection fails on adobe illustrator 2010-07-20 
  rcParams['text.usetex'] = True
  rcParams['ps.useafm'] = False   # use of afm fonts, results in small files
  rcParams['ps.fonttype'] = 3    # Output Type 3 (Type3) or Type 42 (TrueType)

  #mp.rcParams['ps.usedistiller'] = 'xpdf'
  #mp.rcParams['ps.usedistiller'] = 'ghostscript'
  #mp.rcParams['ps.distiller.res'] = 12000
  figure(fig)
  clf()

  panel_width = 1. / n_horz_panels*0.75   # relative width of each subpanel
  hspace = 1. / n_horz_panels*0.25        # horizontal space between subpanels

  panel_height = 1. / n_vert_panels*0.70   # relative height of each subpanel
  vspace = 1. / n_vert_panels*0.25         # vertical space between subpanels

                            # left margin (relative coordinates)
  voffset=voffset_total / n_vert_panels     # bottom margin (relative coordinates)

  return panel_factory(scale, n_horz_panels, n_vert_panels, hoffset, voffset, hspace, vspace, panel_width, panel_height)


# font for labels
font = {'fontname' : 'Helvetica', 'color' : 'k', 'fontweight' : 'bold', 'fontsize' : 24}


# define some gray levels
black = (0.0,0.0,0.0)
darkgray = (0.25, 0.25, 0.25)
midgray = (0.5, 0.5, 0.5)
lightgray = (0.75, 0.75, 0.75)
white = (1.0, 1.0, 1.0)
